{$S+}{$R+}
Unit GraphLib;
Interface
Uses Crt,Dos;
Const
    AllocateOn              = true;
    AllocateOff             = false;
    ErrorFlagOn             = true;
    ErrorFlagOff            = false;
    PictureFon              = true;
    FonPicture              = false;
    Pressed                 = true;
    NotPressed              = false;
    BitMapInfoHeader_Size   = 40;
    BMP_Sign		    = $4d42;
    BitMapFileHeader_Size   = 14;

    ErrorMessages : array[1..7] of string[60]=
                ('Error ( 01 ) : Error of reading file.',
                 'Error ( 02 ) : Cann''t init this mode.',
                 'Error ( 03 ) : GetMem error.',
                 'Error ( 04 ) : This file isn''t BMP file.',
                 'Error ( 05 ) : Coordinates are out of range',
                 'Error ( 06 ) : Unknown error ',
                 'Error ( 07 ) : Error in working with a font');

Type
     gltBytesArray      = array [ 0..$FFFE ] of byte;
     gltBytesPtr        = ^gltBytesArray;
     gltBytesFile       = File of Byte;
     gltBitMap          = record
                           Addr  : gltBytesPtr;
                           D     : word;  { BitMap Width }
                           H     : word;  { BitMap Height }
                          end;

     gltPalette         = array [ 0..767 ] of byte;

     gltPoint           = record
                           x,y : word;
                          end;
     gltTexPoint       = record
                           u,v : word;
                          end;

     gltPointsArray     = array [ 0..$3FFE ] of gltPoint;
     gltPointsPtr       = ^gltPointsArray;
     gltTexPointsArray  = array [ 0..$3FFE ] of gltTexPoint;
     gltTexPointsPtr    = ^gltTexPointsArray;

     gltChar            = record
                            p : gltBytesPtr;
                            w : byte;
                          end;

     gltCharArray       = array [ 0..255 ] of gltChar;
     gltFont            = record
                            H       : byte;
                            FontSize: word;
                            SumbArr : gltCharArray;
                          end;

     Scan               = record
                            XStart, XEnd : word;
                            UStart, UEnd: word;
                            VStart, VEnd: word;
                          end;

     BitMapFileHeader   = record
                            bfType : word;
                            bfSize : longint;
                            bfReserved1 : word;
                            bfReserved2 : word;
                            bfOffBits : longint;
                          end;

     BitMapInfoHeader   = record
                             biSize : longint;
                             biWidth : longint;
                             biHeight : longint;
                             biPlanes : word;
                             biBitCount : word;
                             biCompression : longint;
                             biSizeImage : longint;
                             biXPelsPerMeter : longint;
                             biYPelsPerMeter : longint;
                             biClrUsed : longint;
                             biClrImportant : longint;
                          end;

     RGBQuad            = record
                             rgbBlue  : byte;
                             rgbGreen : byte;
                             rgbRed   : byte;
                             rgbReserved : byte;
                          end;


Var
    Screen    : gltBitMap;
    glError   : word;
    ErrorFlag : boolean;
    ch        : char;
    CurFont   : gltFont;

{ Procedures working with Graph-Text modes }
Procedure glInit320x200;  { Inits graphic mode: 320x200, 256 colors }
Procedure glClose320x200; { Inits text mode: 80x25, 16 colors }

{ Procedures working with a Palette }
Procedure glReadPalette ( FileName : string; var Pal : gltPalette );
           { Reads picture's palette from a file and puts it to Pal }
Procedure glReadBMPPalette ( FileName : string; var Pal : gltPalette );
           { Reads Palette from BMP - file & puts it into Pal -
            the same as glReadpalette but Filename is BMP-file name  }
Procedure glSetPalette ( var Pal : gltPalette );
           { Sets palette which is in Pal }
Procedure glSetFilePalette ( FileName : string );
           { Reads palette from a file and then sets it,
             so glSetFilePalette = glReadPalette + glSetPalette }
Procedure glSetBMPFilePalette ( FileName : string );
           { Reads palette from BMP-file and then sets it }
Procedure glChangePalette ( var Pal : gltPalette; StepsN : word;
                            DelayStop : word; Direction : boolean );

{ Procedures working with a BitMap }
Procedure glReadBitMap ( FileName : string;
                         var BM : gltBitMap; Alloc : boolean  );
Procedure glReadBMPBitMap ( FileName : string;
                            var BM : gltBitMap; Alloc : boolean  );
Procedure glCreateBitMap ( var BM : gltBitMap; Width, Height : word );
Procedure glDeleteBitMap ( var BM : gltBitMap );
Procedure glClearBitMap( var BM : gltBitMap; col : byte );
Procedure glCopyBitMap( var BMDest, BMSourse : gltBitMap );

{ Procedures working with coordinates }
function glWindowCheck( P : gltPointsPtr; N : word;
                        X1, Y1, X2, Y2 : word ) : boolean;
Procedure glTurnCoordinates ( Sourse, Dest : gltPointsPtr; Alfa : real;
                            XC,YC : integer; n : integer );

{ Procedures working with picture output }
Procedure glPutBitMap ( var Target,BM : gltBitMap; x,y : word );
Procedure glPutTexturePolygon ( var Target,BM : gltBitMap;
                                N : byte; P : gltPointsPtr;
                                T : gltTexPointsPtr );


Procedure glPutSolidPolygon ( var Target : gltBitMap;
                                N : byte; P : gltPointsPtr; col : byte );

{ Procedures working with a screen desine }
Procedure glPoint ( var Target : gltBitMap; x1,y1 :word; Col : word );
Procedure glBar( var Target: gltBitMap; X1,Y1,X2,Y2 : word; Col : word );
Procedure glVerticalLine (var Target: gltBitMap;X1,Y1,dl : word;
                           Col : word ) ;
Procedure glHorizontalLine (var Target: gltBitMap;X1,Y1,dl : word;
                           Col : word ) ;
Procedure glLine ( var Target : gltBitMap;X1,Y1,X2,Y2 : integer;
                           Col : word );
Procedure glFrame ( var Target: gltBitMap; X1,Y1,X2,Y2 : word; Col : word );

Procedure glVolumeButton ( Target :gltBitMap; X1,Y1,X2,Y2 : word; ButtonColor,
                       Color1, Color2 : word; Pressed : boolean );
Procedure glWindow ( Target : gltBitMap; var P : gltBytesPtr; x1,y1,x2,y2 : word;
                      WindowColor,ShadowColor,FrameColor : word );

Procedure glDeleteWindow ( Target: gltBitMap; var P : gltBytesPtr;
                           x1,y1,x2,y2 : word );

Procedure glPutChar ( Target : gltBitMap; X,Y : word;
                      c : char; Col : word );
procedure glPutString( Target : gltBitMap; X, Y : word; s : string;
                       col : byte );

{ Procedures working with Fonts }
Procedure glReadFont( FileName : string; var Font : gltFont;
                      Alloc : boolean );

Procedure glSetFont ( var Font : gltFont );

Function glPixStrLength ( st : string ) : word;

Procedure glDeleteFont ( var Font : gltFont );

Function glReadStr ( Target : gltBitMap; XStart,Ystart,XLim,YLim : word;
                     TextColor : word ) : string ;

{ Procedure working with Grahpics Errors }
Procedure glSetErrorFlag ( Flag : boolean );

{-----------------------------------------------------}
Implementation

Procedure ErrorWorker( n : word );
  begin
    glError := n;
    if ErrorFlag
      then
        begin
          glClose320x200;
          writeln( ErrorMessages[ n ] );
          halt( n );
        end;
  end;

{ Procedures working with Graph-Text modes }
Procedure glInit320x200;
Var Possible : byte;
begin
 asm
    MOV AH,0
    MOV AL,13H      { graphmode is used ( 320*200 & 256 colours )  }
    INT 10H
    jnc @@NoError
    MOV Possible,1
  @@NoError :
    MOV Possible,0
 end;
 If Possible = 1 then ErrorWorker( 2 );
end;  { of glInit320x200 procedure }


Procedure glClose320x200; assembler;
 asm
   MOV AH,0     { Textmode is used now }
   MOV AL,3
   INT 10H
 end;


{----------------------------------------------------------}
{ Procedures working with a Palette }
Procedure glReadPalette ( FileName : string; var Pal : gltPalette );
 var PalFile   : file;
     Tmp       : byte;
     i         : longint;
     j         : longint;
     FSize     : longint;

 begin
  {$I-}
  assign ( PalFile,FileName );
  reset ( PalFile, 1 );
  if Ioresult <> 0 then ErrorWorker( 1 );
  blockread( PalFile, Pal, 256*3 );
  if Ioresult <> 0 then ErrorWorker( 1 );
  {$I+}
  close( PalFile );
 end;  { of glReadPalette procedure }


Procedure glReadBMPPalette ( FileName : string; var Pal : gltPalette );
 var BMPHead   : BITMAPFILEHEADER;
     BMPInfo   : BITMAPINFOHEADER;
     RGBInfo   : RGBQUAD;
     BMPFile   : file;
     i         : word;
 begin
  assign( BMPFile , FileName );
  {$I-}
  reset( BMPFile , 1 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( BMPFile , BMPHead , sizeof( BITMAPFILEHEADER ) );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( BMPFile , BMPInfo , sizeof( BITMAPINFOHEADER ) );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  if BMPHead.bfType <> BMP_SIGN
    then
      ErrorWorker( 4 );
  for i := 0 to 255 do
    begin
      blockread( BMPFile , RGBInfo , sizeof( RGBQUAD ) );
      if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
      Pal[ i*3 ] := RGBInfo.rgbRed shr 2 ;
      Pal[ i*3 + 1 ] := RGBInfo.rgbGreen shr 2;
      Pal[ i*3 + 2 ] := RGBInfo.rgbBlue shr 2;
    end;
  { glSetPalette ( @Pal );}
  {$I+}
 end; { of glReadBMPPalette procedure }

Procedure glSetPalette ( var Pal : gltPalette );
 var i : byte;
 begin
  while ( ( Port[ $3da ] and $8 ) <> 0 ) do ;
  while ( not ( ( Port[ $3da ] and $8 ) <> 0 ) ) do ;
  for i := 0 to 255 do
    begin
      Port[ $3c8 ] := i;
      Port[ $3c9 ] :=  Pal[ i*3 ] ;
      Port[ $3c9 ] :=  Pal[ i*3 + 1 ] ;
      Port[ $3c9 ] :=  Pal[ i*3 + 2 ] ;
    end;
 end; { of glSetPalette procedure }

Procedure glSetFilePalette ( FileName : string );
 var PalFile   : file;
     Tmp       : byte;
     i         : longint;
     j         : longint;
     FSize     : longint;
     Pal       : gltPalette;

 begin
  {$I-}
  assign ( PalFile,FileName );
  reset ( PalFile, 1 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( PalFile, Pal, 256*3 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  {$I+}
  close( PalFile );
  glSetPalette( Pal );
 end; { of glSetFilePalette procedure }

Procedure glSetBMPFilePalette ( FileName : string );
 var BMPHead   : BITMAPFILEHEADER;
     BMPInfo   : BITMAPINFOHEADER;
     RGBInfo   : RGBQUAD;
     BMPFile   : file;
     i         : word;
     Pal       : gltPalette;
 begin
  assign( BMPFile , FileName );
  {$I-}
  reset( BMPFile , 1 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( BMPFile , BMPHead , sizeof( BITMAPFILEHEADER ) );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( BMPFile , BMPInfo , sizeof( BITMAPINFOHEADER ) );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  if BMPHead.bfType <> BMP_SIGN
    then
      ErrorWorker( 4 );
  for i := 0 to 255 do
    begin
      blockread( BMPFile , RGBInfo , sizeof( RGBQUAD ) );
      if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
      Pal[ i*3 ] := RGBInfo.rgbRed shr 2 ;
      Pal[ i*3 + 1 ] := RGBInfo.rgbGreen shr 2;
      Pal[ i*3 + 2 ] := RGBInfo.rgbBlue shr 2;
    end;
   glSetPalette ( Pal );
  {$I+}
 end; { of glSetBMPFilePalette procedure }

Procedure glChangePalette ( var Pal : gltPalette; StepsN : word;
                            DelayStop : word; Direction : boolean );
 var RPal     : array [ 0..767 ] of real;
     Steps    : array [ 0..767 ] of real;
     i        : integer;
     j        : integer;
 begin
 if Direction = FonPicture then
   for i := 0 to 767 do
    begin
     RPal[ i ] := 0;
     Steps[ i ] := Pal[ i ] / StepsN;
    end
 else
  begin
   for i := 0 to 767 do
    begin
      RPal[ i ] := Pal[ i ];
      Steps[ i ] := - Pal[ i ]/ StepsN;
     end;
  end;
 for j := 0 to StepsN-1 do
  begin
    while ( ( Port[ $3da ] and $8 ) <> 0 ) do ;
    while ( not ( ( Port[ $3da ] and $8 ) <> 0 ) ) do ;
    for i := 0 to 255 do
      begin
        Port[ $3c8 ] := i;
        Port[ $3c9 ] :=  round( RPal[ i*3 ] );
        Port[ $3c9 ] :=  round( RPal[ i*3 + 1 ] );
        Port[ $3c9 ] :=  round( RPal[ i*3 + 2 ] );
        RPal [ i*3 ] := RPal[ i*3 ] + Steps [ i*3 ];
        RPal [ i*3+1 ] := RPal[ i*3+1 ] + Steps [ i*3+1 ];
        RPal [ i*3+2 ] := RPal[ i*3+2 ] + Steps [ i*3+2 ];
      end;
    Delay ( DelayStop );
   end;

 end;  { of glChangePalette procedure }

{--------------------------------------------------------------------}
function glWindowCheck( P : gltPointsPtr; N : word;
                        X1, Y1, X2, Y2 : word ) : boolean;
 var i : word;
 begin
   glWindowCheck := false;
   for i := 0 to N-1 do
     begin
       if (P^[ i ].x <= X1) or (P^[ i ].x >= X2) or
          (P^[ i ].y <= Y1) or (P^[ i ].y >= Y2)
         then
           exit;
     end;
   glWindowCheck := true;
 end;

{ Procedures working with a BitMap }

Procedure glReadBitMap ( FileName : string; var BM : gltBitMap;
                         Alloc : boolean );
 var TmpLength : longint;
     TmpLength1: longint;
     j         : longint;
     DatFile   : file;
     Tmp       : byte;
     w,hg      : word;
Begin
  {$I-}
  assign ( DatFile,FileName );
  reset ( DatFile, 1 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread ( DatFile, TmpLength, 4 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  w :=  word( TmpLength  and $00FF );
  blockread ( DatFile, TmpLength, 4 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  hg := word( TmpLength and $00FF );
  if not Alloc then
    glDeleteBitMap( BM );
  glCreateBitMap( BM, w, hg );
  blockread( DatFile, BM.Addr^, w*hg );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  close ( DatFile );
  {$I+}
 end;  { of glReadBitMap procedure }

Procedure glReadBMPBitMap ( FileName : string; var BM : gltBitMap;
                            Alloc : boolean );
 var BMPHead   : BITMAPFILEHEADER;
     BMPInfo   : BITMAPINFOHEADER;
     RGBInfo   : RGBQUAD;
     BMPFile   : file;
     i         : word;
     PalPtr    : gltBytesPtr;
     D         : byte;
     BufTmp    : longint;
 begin
  assign( BMPFile , FileName );
  {$I-}
  reset( BMPFile , 1 );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( BMPFile , BMPHead , sizeof( BITMAPFILEHEADER ) );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
  blockread( BMPFile , BMPInfo , sizeof( BITMAPINFOHEADER ) );
  if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;

  if BMPHead.bfType <> BMP_SIGN
    then
      ErrorWorker( 4 );
  for i := 0 to 255 do
    begin
      blockread( BMPFile , RGBInfo , sizeof( RGBQUAD ) );
      if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
    end;
  D := BMPInfo.biWidth;
  D := D and 3;
  if D > 0 then D := 4 - D;

  if not Alloc then glDeleteBitMap( BM );
  glCreateBitMap( BM, BMPInfo.biWidth,  BMPInfo.biHeight);

  for i := 0 to BMPInfo.biHeight - 1  do
   begin
    BlockRead ( BMPFile , BM.Addr^[(BMPInfo.biHeight - i - 1) *
          BMPInfo.biWidth]  ,BMPInfo.biWidth );
    if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
    if D > 0
      then
        begin
          BlockRead ( BMPFile, BufTmp,D );
          if Ioresult <> 0 then begin ErrorWorker( 1 ); exit; end;
        end;
   end;

  close( BMPFile );
 {I+}
 end; { of glReadBMPBitMap procedure }

Procedure glCreateBitMap ( var BM : gltBitMap; Width,Height : word );
 begin
   with BM do
    begin
     D := Width;
     H := Height;
     GetMem ( Addr,D*H );
    end;
 end; { of  glCreateBitMap procedure }

Procedure glDeleteBitMap ( var BM : gltBitMap );
 begin
   if BM.Addr = nil then exit;
   FreeMem ( BM.Addr, BM.D * BM.H );
 end; { of  glCreateBitMap procedure }

Procedure glClearBitMap( var BM : gltBitMap; col : byte );
  begin
    FillChar( BM.Addr^, BM.H*BM.D, col );
  end;

Procedure glCopyBitMap( var BMDest, BMSourse : gltBitMap );
  var N : word;
  begin
    if ( BMDest.D <> BMSourse.D ) or ( BMDest.H <> BMSourse.H )
      then
        ErrorWorker( 4 );
    N := BMDest.D * BMDest.H;
    asm
      push DS
      push ES

      lds  SI, BMDest
      mov  AX, [SI+2]
      mov  SI, [SI]
      mov  ES, AX

      lds  DI, BMSourse
      mov  AX, [DI+2]
      mov  DI, [DI]
      mov  DS, AX

      mov  CX, N
      mov  BX, CX
      sar  CX, 1
      and  BX, 1
      rep  movsw
      test BX, BX
      jz   @@NoOneByte
      movsb
@@NoOneByte :
      pop  ES
      pop  DS
    end;
  end;

{------------------------------------------------------------------}
{ Procedures working with picture output }

Procedure glPutBitMap ( var Target, BM : gltBitMap; x,y : word );
 var i          : longint;
     DestOffs   : word;
     DSS, ESS   : word;
     AddOffs    : word;
     BitMapH, BitMapD : word;
     OffsS, OffsD, SegS, SegD : word;
begin
 if ( X > Target.D ) or ( Y > Target.H ) then
   ErrorWorker ( 5 );
 DestOffs := ( Target.D * Y ) + X;
 AddOffs := Target.D - BM.D;
 BitMapH := BM.H;
 BitMapD := BM.D;
 OffsD := ofs( Target.Addr^ );
 SegD := seg( Target.Addr^ );
 OffsS := ofs( BM.Addr^ );
 SegS := seg( BM.Addr^ );
 asm
   mov BX, BitMapH
   mov DX, AddOffs
   mov DI, OffsD
   mov SI, OffsS

   mov AX, ES
   mov ESS, AX
   mov AX, SegD
   mov ES, AX

   mov AX, DS
   mov DSS, AX
   mov AX, SegS
   mov DS, AX

   add DI, DestOffs

   cmp BX, 0
   jle @@EndLoop

@@StartLoop:
   mov CX, BitMapD
   rep movsb
   add DI, DX
   dec BX

   cmp BX, 0
   jg  @@StartLoop

@@EndLoop :

   mov AX, ESS
   mov ES, AX
   mov AX, DSS
   mov DS, AX
 end;
end;

Procedure glPutTexturePolygon  ( var Target, BM : gltBitMap;
                                 N : byte; P : gltPointsPtr;
                                 T : gltTexPointsPtr );
 var   MaxY,MinY           : word;
       i,j,tmp             : word;
       y,dx,dy             : word;
       TmpOffs,Offs        : word;
       x1,x2               : integer;
       y1,y2               : integer;
       u1,u2,UTmp          : integer;
       v1,v2,VTmp          : integer;
       EndIndex,StartIndex : integer;
       StepX,StepU,StepV   : real;
       u,v,x               : real;
       ScanArray           : array [0..199] of Scan;
       SourceBuff, DestBuff: gltBytesPtr;

 begin
   if (not glWindowCheck( P, N, 0, 0, Target.D-1, Target.H-1)) or
      (not glWindowCheck( gltPointsPtr(T), N, 0, 0, BM.D-1, BM.H-1))
      then
        ErrorWorker(5);

   MinY := P^[0].y;
   MaxY := MinY;

   for i:= 1 to n-1 do
     begin                     { Y minimum & Y maximum are derived }
       tmp := P^[i].y;
       if tmp < MinY then
         MinY := tmp
       else
         if tmp > MaxY then
           MaxY := tmp;
     end;

   EndIndex := n-1;
   StartIndex := 0;

   while StartIndex < n do
    begin
      if P^[ StartIndex ].y = P^[ EndIndex ].y then
         begin
           EndIndex := StartIndex;
           inc( StartIndex );
           continue;
         end;

      if P^[ StartIndex ].y < P^[ EndIndex ].y
        then
         begin
           x1 := P^[ StartIndex ].x;
           x2 := P^[ EndIndex ].x;
           y1 := P^[ StartIndex ].y;
           y2 := P^[ EndIndex ].y;
           u1 := T^[ StartIndex ].u;
           u2 := T^[ EndIndex ].u;
           v1 := T^[ StartIndex ].v;
           v2 := T^[ EndIndex ].v;
         end
        else
         begin
           x2 := P^[ StartIndex ].x;
           x1 := P^[ EndIndex ].x;
           y2 := P^[ StartIndex ].y;
           y1 := P^[ EndIndex ].y;
           u2 := T^[ StartIndex ].u;
           u1 := T^[ EndIndex ].u;
           v2 := T^[ StartIndex ].v;
           v1 := T^[ EndIndex ].v;
         end;

      Dy :=  Y2 - Y1 ;
      StepX :=  (x2 - x1)  / Dy;
      StepU :=  (u2 - u1)  / Dy;
      StepV :=  (v2 - v1 ) / Dy;

      u := u1;
      v := v1;
      x := x1;

      for y := y1 to y2-1 do
        begin
          ScanArray[ y ].XStart := ScanArray[ y ].XEnd;
          ScanArray[ y ].UStart := ScanArray[ y ].UEnd;
          ScanArray[ y ].VStart := ScanArray[ y ].VEnd;

          ScanArray[ y ].XEnd := round( X );
          ScanArray[ y ].UEnd := round( U );
          ScanArray[ y ].VEnd := round( V );

          x := StepX + x;
          u := StepU + u;       { ScanArray creating }
          v := StepV + v;
         end;

      EndIndex := StartIndex;
      inc( StartIndex );
    end;                    { of while }

    offs := MinY * Target.D;
    DestBuff := Target.Addr;
    SourceBuff := BM.Addr;

    for y := MinY to MaxY-1 do
      begin
        if ScanArray[ y ].XStart > ScanArray[ y ].XEnd then
           begin
             tmp := ScanArray[ y ].XStart;
             ScanArray[ y ].XStart := ScanArray[ y ].XEnd;
             ScanArray[ y ].XEnd := tmp;

             tmp := ScanArray[ y ].UStart;
             ScanArray[ y ].UStart := ScanArray[ y ].UEnd;
             ScanArray[ y ].UEnd := tmp;

             tmp := ScanArray[ y ].VStart;
             ScanArray[ y ].VStart := ScanArray[ y ].VEnd;
             ScanArray[ y ].VEnd := tmp;
           end;

        Dx := ScanArray[ y ].XEnd - ScanArray[ y ].XStart;
        inc ( Dx );
        StepU := ( ScanArray[ y ].UEnd - ScanArray[ y ].UStart ) / Dx;
        StepV := ( ScanArray[ y ].VEnd - ScanArray[ y ].VStart ) / Dx;
        U := ScanArray[ y ].UStart;
        V := ScanArray[ y ].VStart;
        TmpOffs := offs + ScanArray[ y ].XStart;

        for i := 0 to ( Dx-1 ) do
          begin
            UTmp := round ( U );
            VTmp := round ( V );
            DestBuff^[ TmpOffs ] :=
                SourceBuff^[ (VTmp * BM.D) + UTmp ];
            U := U + StepU;
            V := V + StepV;
            inc ( TmpOffs );
          end;

        inc ( Offs, Target.D );
      end;
 end; { of glPutTexturePolygon procedure }

Procedure glTurnCoordinates ( Sourse, Dest : gltPointsPtr;
                            Alfa : real; XC,YC : integer; n : integer );
var
   CosAlfa     : real;
   SinAlfa     : real;
   i           : integer;
begin

 CosAlfa := cos ( Alfa );
 SinAlfa := sin ( Alfa );
 for i := 0  to n-1 do
  begin
    Dest^[i].x :=  ( round( ( Sourse^[i].x - XC ) *  CosAlfa )
                  -  round( ( Sourse^[i].y - YC ) *  SinAlfa ) ) + XC;
    Dest^[i].y :=  ( round( ( Sourse^[i].x - XC ) *  SinAlfa )+
                 round( ( Sourse^[i].y - YC ) *  CosAlfa ) ) + YC;
  end;
end; { of TurnCoordinates procedure }

Procedure glPutSolidPolygon ( var Target : gltBitMap;
                                N : byte; P : gltPointsPtr; col : byte );
 var   MaxY,MinY           : word;
       i,j,tmp             : word;
       y,dx,dy             : word;
       TmpOffs,Offs        : word;
       x1,x2               : integer;
       y1,y2               : integer;
       EndIndex,StartIndex : integer;
       StepX               : real;
       x                   : real;
       ScanArray           : array [0..199] of Scan;
       DestBuff            : gltBytesPtr;

 begin
   if (not glWindowCheck( P, N, 0, 0, Target.D-1, Target.H-1))
      then
        ErrorWorker(5);

   MinY := P^[0].y;
   MaxY := MinY;

   for i:= 1 to n-1 do
     begin                     { Y minimum & Y maximum are derived }
       tmp := P^[i].y;
       if tmp < MinY then
         MinY := tmp
       else
         if tmp > MaxY then
           MaxY := tmp;
     end;

   EndIndex := n-1;
   StartIndex := 0;

   while StartIndex < n do
    begin
      if P^[ StartIndex ].y = P^[ EndIndex ].y then
         begin
           EndIndex := StartIndex;
           inc( StartIndex );
           continue;
         end;

      if P^[ StartIndex ].y < P^[ EndIndex ].y
        then
         begin
           x1 := P^[ StartIndex ].x;
           x2 := P^[ EndIndex ].x;
           y1 := P^[ StartIndex ].y;
           y2 := P^[ EndIndex ].y;
         end
        else
         begin
           x2 := P^[ StartIndex ].x;
           x1 := P^[ EndIndex ].x;
           y2 := P^[ StartIndex ].y;
           y1 := P^[ EndIndex ].y;
         end;

      Dy :=  Y2 - Y1 ;
      StepX :=  (x2 - x1)  / Dy;

      x := x1;

      for y := y1 to y2-1 do
        begin
          ScanArray[ y ].XStart := ScanArray[ y ].XEnd;
          ScanArray[ y ].XEnd := round( X );
          x := StepX + x;
         end;

      EndIndex := StartIndex;
      inc( StartIndex );
    end;                    { of while }

    offs := MinY * Target.D;
    DestBuff := Target.Addr;

    for y := MinY to MaxY-1 do
      begin
        if ScanArray[ y ].XStart > ScanArray[ y ].XEnd then
           begin
             tmp := ScanArray[ y ].XStart;
             ScanArray[ y ].XStart := ScanArray[ y ].XEnd;
             ScanArray[ y ].XEnd := tmp;
           end;

        Dx := ScanArray[ y ].XEnd - ScanArray[ y ].XStart;
        inc ( Dx );
        TmpOffs := offs + ScanArray[ y ].XStart;

        for i := 0 to ( Dx-1 ) do
          begin
            DestBuff^[ TmpOffs ] := col;
            inc ( TmpOffs );
          end;

        inc ( Offs, Target.D );
      end;
 end; { of glPutTexturePolygon procedure }

{-----------------------------------------------------------------}

{ Procedures working with screen desine }

Procedure glPoint ( var Target : gltBitMap; x1,y1 : word; Col : word );
 begin
   if ( X1 > Target.D ) or ( Y1 > Target.H ) then
    ErrorWorker ( 5 );
   Target.Addr^[ y1 * Target.D + x1 ] := Col;
 end; { of glPoint }

Procedure glBar( var Target: gltBitMap;X1,Y1,X2,Y2 : word; Col : word );
 var i,j          : word;
     ScreenOffs   : word;
     Offs         : word;
 begin
   if ( X1 > Target.D ) or ( Y1 > Target.H ) or
      ( X2 > Target.D ) or ( Y2 > Target.H )  then
     ErrorWorker ( 5 );
   ScreenOffs := Y1 * Target.D;
   for j := Y1 to Y2-1 do
    begin
     Offs := ScreenOffs+x1;
     for i := X1 to X2 do
      begin
       Target.Addr^[ Offs ] := Col ;
       inc ( Offs );
      end;
     inc (ScreenOffs,320 );
    end;
 end;      { of procedure }

Procedure glVerticalLine (var Target: gltBitMap;X1,Y1,dl : word;
                             Col : word ) ;
 var size,j       : word;
     ScreenOffs   : word;
     Offs         : word;
begin
 if ( X1 > Target.D ) or ( Y1 + dl > Target.H ) then
    ErrorWorker ( 5 );
 ScreenOffs := Y1 * Target.D;
 Offs := ScreenOffs + X1;
 size := Y1 + dl;
 for j := Y1 to size do
  begin
   Target.Addr^[ Offs ] := Col;
   inc ( Offs , Target.D );
  end;
end;  { of glVerticalLine procedure }

Procedure glHorizontalLine (var Target: gltBitMap;X1,Y1,dl : word;
                              Col : word ) ;
begin
 if ( X1+dl > Target.D ) or ( Y1 > Target.H ) then
    ErrorWorker ( 5 );
  FillChar( Target.Addr^[ Y1 * Target.D + X1 ], dl, Col );
end;  { of glHorizontalLine procedure }

Procedure glFrame (var Target: gltBitMap;X1,Y1,X2,Y2 : word; Col : word );
begin
  if ( X1 > Target.D ) or ( Y1 > Target.H ) or
    ( X2 > Target.D ) or ( Y2 > Target.H )then
   ErrorWorker ( 5 );
  glHorizontalLine ( Target,X1,Y1,X2-X1,Col );
  glVerticalLine   ( Target,X2,Y1,Y2-Y1,Col );
  glHorizontalLine ( Target,X1,Y2,X2-X1,Col );
  glVerticalLine   ( Target,X1,Y1,Y2-Y1,Col );

end; { of glFrame procedure }

Procedure glVolumeButton ( Target :gltBitMap;X1,Y1,X2,Y2 : word ;
                   ButtonColor,Color1,Color2 : word; Pressed : boolean );
 begin
   if ( X1 > Target.D ) or ( Y1 > Target.H ) or
    ( X2 > Target.D ) or ( Y2 > Target.H )then
    ErrorWorker ( 5 );
   If Pressed then
    begin
     Color1 := Color2 + Color1;
     Color2 := Color1 - Color2;
     Color1 := Color1 - Color2;
    end;

   glBar ( Target,X1,Y2,X2,Y2,ButtonColor );
   glHorizontalLine( Target,X1,Y1,X2-X1,Color2 );
   glVerticalLine( Target,X2,Y1,Y2-Y1 ,Color1 );
   glHorizontalLine ( Target,X1,Y2,X2-X1,Color1 );
   glVerticalLine ( Target,X1,Y1,Y2-Y1,Color2 );
 end;

Procedure glWindow ( Target : gltBitMap; var P : gltBytesPtr; x1,y1,x2,y2 : word;
                      WindowColor,ShadowColor,FrameColor : word );
 var
      i          :  word;
      j          :  word;
      TmpY1,TmpY2:  word;
      TmpX1,TmpX2:  word;
      W,D        :  word;
      WindowH    :  word;
      WindowW    :  word;
      StartIndex,EndIndex : word;
begin
  GetMem ( P,( x2 - x1 + 10 ) * ( y2 - y1 + 10) );
  TmpX1 := X1;
  TmpX2 := X2 + 20;
  TmpY1 := Y1;
  TmpY2 := Y2 + 20 ;
 { WindowH := y2 - y1;
  WindowW := x2 - x1;

  for i:= 0 to WindowH do
    move ( Target.Addr^,P^,WindowW);}
  StartIndex := Target.D * ( Y1- 10 ) + ( x1 - 5 );
  EndIndex   := Target.D * (Y2 +10 ) +  ( x2 + 5 );
   for i:= StartIndex to EndIndex do
     P^ [ i ] := Target.Addr^[ i ];
  glBar ( Target,X1+2,Y1+2,X2-2,Y2-1 ,WindowColor );
  glBar ( Target,X1 + 10,Y2 + 1,X2 +5,Y2 + 5 ,22);
  glBar ( Target,X2 + 1,Y1 + 5,X2 + 5, Y2 + 5 ,22 );
  glFrame ( Target,X1,Y1,X2,Y2 ,FrameColor );
  glFrame ( Target,X1+1,Y1+1,X2-1,Y2-1,FrameColor );
end;  { of glWindow procedure }

Procedure glDeleteWindow (  Target: gltBitMap; var P : gltBytesPtr;
                           x1,y1,x2,y2 : word );
var   WindowH    :  word;
      WindowW    :  word;
      i          :  word;
      TmpY1,TmpY2:  word;
      TmpX1,TmpX2:  word;
      StartIndex,EndIndex : word;

begin
  WindowH := y2 - y1;
  WindowW := x2 - x1;

  {for i:= 0 to WindowH do
    move ( P^,Target.Addr^,WindowW);  }
   StartIndex := Target.D * ( Y1- 10 ) + ( x1  - 5 );
   EndIndex   := Target.D * (Y2+ 10 ) + ( x2 + 5 );
   for i:= StartIndex to EndIndex do
    Target.Addr^[ i ] := P^ [ i ];
 if ( P <> nil) and (SSeg <>0 ) then
    FreeMem ( P, ( x2 - x1 + 10 ) * ( y2 - y1 + 10) );
 end;  { of glDeleteWindow procedure }

Procedure glLine ( var Target : gltBitMap;X1,Y1,X2,Y2 : integer;
                           Col : word );
var offs          : word;
    dx,dy,dx2,dy2 : integer;
    doffs         : integer;
    i,tmp         : word;
    addr          : gltBytesPtr;
    e             : integer;
 begin
   if ( X1 > Target.D ) or ( Y1 > Target.H ) or
    ( X2 > Target.D ) or ( Y2 > Target.H )then
    ErrorWorker ( 5 );
   if X1 = X2 then
    begin
       if Y1 > Y2
         then
           glVerticalLine ( Target,x2,y2,y1-y2+1,col )
         else
           glVerticalLine ( Target,x1,y1,y2-y1+1,col );
       exit;
    end;
   if Y1 = Y2 then
    begin
       if X1 > X2
         then
           glHorizontalLine ( Target,x2,y2,x1-x2+1,col )
         else
           glHorizontalLine ( Target,x1,y1,x2-x1+1,col );
       exit;
    end;
  addr := Target.addr;
  dy := y2-y1;
  dx := x2-x1;
  if  abs ( dx ) > abs ( dy ) then
   begin
    if dx < 0 then
     begin
       dx := -dx;
       dy := -dy;
       offs := y2 * target.D + x2;
     end
    else
     offs := y1 * target.D + x1;
    dy2 := dy shl 1;
    dx2 := dx shl 1;
    e := dy2 - dx;
    if dy > 0  then
     doffs := Target.D
    else
     doffs := -Target.D;
    for i := 0 to dx do
     begin
       addr^[ offs ] := col;
       inc ( offs );
       if ( e > 0 ) then
        begin
          inc  ( offs, doffs );
          dec ( e,dx2 );
        end;
       inc ( e, abs(dy2) );
     end;
   end
  else
   begin
     if dy < 0 then
       begin
        dx := -dx;
        dy := -dy;
        offs := y2 * Target.D + x2;
       end
     else
      offs := y1 * target.D + x1;
     dy2 := dy shl 1;
     dx2 := dx shl 1;
     e := dx2 - dy;
     if dx > 0 then
      doffs := 1
     else
      doffs := -1;
     for i := 0 to dy do
      begin
       addr^[ offs ] := col;
       inc ( offs,Target.D );
       if ( e > 0 ) then
         begin
           inc ( offs,doffs );
           dec ( e,dy2 );
         end;
       inc ( e,abs(dx2) );
      end;
   end;
 end; { of glLine procedure }

Procedure glPutChar ( Target : gltBitMap; X,Y : word;
                      c : char; Col : word );
 var i        : integer;     {BVS : Change parameters}
     j        : integer;
     Offs     : word;
     YOffs    : word;
     ByteOffs : word;
     D        : word;
     CharH    : byte;
     CharW    : byte;
     Simbol   : gltChar;
     PS       : gltBytesPtr;
begin
   CharH := CurFont.H;
   Simbol := CurFont.SumbArr[ ord (c) ];
   CharW := Simbol.W;
   if ( X + CharW > Target.D ) or ( Y+CharH > Target.H ) then
      ErrorWorker(5);       { You're very clever, I see. Thats great ! }
   PS := Simbol.P;
   D := Target.D;
   ByteOffs := 0;
   YOffs := Y * D + X;
   for i := 1 to CharH do
    begin
      Offs := YOffs;
      for j := 1 to CharW do
       begin
        if PS^[ ByteOffs ] > 0 then
           Target.Addr^[ Offs ] := Col;
        inc ( ByteOffs );
        inc ( Offs );
       end;
      inc ( YOffs,D );
    end;
 end; { of glPutChar procedure }

procedure glPutString( Target : gltBitMap; X, Y : word; s : string;
                       col : byte );
  var XCur, i : byte;
  begin
    XCur := X;
    for i := 1 to length(s) do
      begin
        glPutChar( Target, XCur, Y, s[i], col );
        inc( XCur, CurFont.SumbArr[ord(s[i])].w+1 );
      end;
  end;

Procedure glReadFont( FileName : string; var Font : gltFont;
                      Alloc : boolean );
var  i         : integer;
     FontFile  : file;
     SumbSize  : integer;
     P         : gltBytesPtr;
 begin
   {$I-}
   assign ( FontFile, FileName );
   reset ( FontFile,1 );
   if IOResult <> 0 then
     ErrorWorker ( 1 );
   with Font do
    begin
      BlockRead ( FontFile, H, SizeOf( H ) );
      if IOResult <> 0 then ErrorWorker ( 1 );
      SumbSize := 0;
      for i := 0 to 255 do
        begin
          BlockRead ( FontFile, SumbArr[i].w, SizeOf(SumbArr[i].w) );
          inc( SumbSize, SumbArr[i].w );
          if IOResult <> 0 then ErrorWorker ( 1 );
        end;

      SumbSize := SumbSize * H;
      if not Alloc then
        glDeleteFont( Font );
      GetMem ( P, SumbSize );


      BlockRead ( FontFile, P^, SumbSize );
      if IOResult <> 0 then ErrorWorker ( 1 );
      FontSize := SumbSize;

      SumbSize := 0;
      for i := 0 to 255 do
        begin
          SumbArr[i].p := @(P^[ SumbSize ]);
          inc(SumbSize, SumbArr[i].w * H );
        end;
    end;
   {$I+}
   close ( FontFile );
 end;     {of glReadFont procedure }


Procedure glDeleteFont ( var Font : gltFont );
var  i      : integer;
     Height : byte;
 begin
  with Font do
   begin
     if FontSize = 0 then exit;
     FreeMem ( SumbArr[0].P, FontSize );
     FontSize := 0;
   end;
 end;   { of glDeleteFont procedure }

Function glReadStr ( Target : gltBitMap; XStart,Ystart,XLim,YLim : word;
                     TextColor : word ) : string ;

var stVar   : string;
    curX : word;
    i    : word;
 begin
   if CurFont.FontSize = 0 then
     ErrorWorker ( 7 );
   if ( XStart > Target.D ) or ( YStart > Target.H ) or
      ( XLim   > Target.D ) or ( YLim   > Target.H ) then
     ErrorWorker ( 5 );
   i := 1;
   StVar := '                                                 ';
   ch := readkey;
   curX := XStart;
   while ( ch <> #27 ) and ( ch <> #13 ) do
    begin
      glPutChar ( Target,curX,YStart,ch,TextColor );
      stVar[ i ] := ch;
      inc ( i );
      inc ( CurX,9 );
      if CurX > XLim then
       begin
         inc ( YStart,10 );
         CurX := XStart;
       end;
       if YStart > YLim then
         exit;
      ch := readkey;
    end;
    if ch = #27 then
      begin
        glReadStr := '';
      end ;
    if ch = #13 then
      begin
        glReadStr := stVar;
        glReadStr[ i ] := ' ';
      end;

 end; { of glReadStr function  }

Procedure glSetFont ( var Font : gltFont );
 begin
   CurFont := Font;
 end;   { of glSeFont}

Function glPixStrLength ( st : string ) : word;
var  i          : word;
     Size       : word;
     CurLength  : integer;
 begin
   Size := Length ( st );
   CurLength := 0;
   with CurFont do
    begin
     if FontSize = 0 then
      ErrorWorker ( 7 );
     for i := 0 to Size do
       CurLength := CurLength + SumbArr[ ord (st [i] ) ].w ;
    end;
   glPixStrLength := CurLength;
 end; { of glPixStrLength }


{BVS: procedure glSetFont( var fnt : gltFont ); }  {  written }
{BVS: function  glPixStrLength( str : string ) : word; } { written }


{-----------------------------------------------------------------}
{ Procedure working with Grahpics Errors }
Procedure glSetErrorFlag ( Flag : boolean );
 begin
   ErrorFlag := Flag;
 end; { of glSetErrorFlag procedure }


{-----------------------------------------------------------------}
{ Function working with Heap }
Function HeapFunc ( Size : word ): integer; far;
 begin
  if Size > 0
    then
      ErrorWorker( 3 );
  HeapFunc := 1;
 end; { of HeapFunc function }

{-----------------------------------------------------------------}
Begin
 ErrorFlag := ErrorFlagOn;
 glError := 0;
 HeapError := @HeapFunc;
 with Screen do
  begin
    Addr :=Ptr( $A000,$0000 );  { Points on the screen }
    D := 320;
    H := 200;
  end;
End.